##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of Bling.
#
# Bling is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bling is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bling; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
from StringIO import StringIO
from OFS.ObjectManager import BadRequestException
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.atapi import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.CMFCore.DirectoryView import addDirectoryViews, \
     registerDirectory, manage_listAvailableDirectories, \
     manage_addDirectoryView
import os

from Products.Bling import config

# This is the current list of depends
resources = [
    {'id': 'tiny_mce/tiny_mce.js', 'compression' : 'none'},
    {'id': 'json.js'},
    {'id': 'behaviour.js'},
    {'id': 'prototype.js'},
    {'id': 'builder.js'},
    {'id':'effects.js'},
    {'id':'dragdrop.js'},
    {'id':'controls.js', 'compression': 'none'},
    {'id': 'slider.js'},
    {'id': 'bling.js'},
]



def update_registries(self, out):
    # Register two sets of javascripts, the global sequence of
    # libraries, and the behaviors specific to our particular
    # search application
    portal_js = self.portal_javascripts
    existing = portal_js.getResourceIds()

    rrversion = self.portal_quickinstaller.getProductVersion('ResourceRegistries')

    for resource in resources:
        if resource.get('id') not in existing:
            # version handling
            if rrversion < "1.2": resource.pop('compression', None)
            portal_js.registerScript(**resource)
    # Register the CSS too
    portal_css = self.portal_css
    if 'bling.css' not in portal_css.getResourceIds():
        portal_css.manage_addStylesheet(id = 'bling.css', enabled=True)


def update_skins(self, out):
    st = getToolByName(self, 'portal_skins')
    manage  = st.manage_skinLayers

    # We want to add the bling skin to all
    # in the skin path
    # and create an ajax skin with the special
    # limited templates for ajax
    # and assign it to a specific layer
    if config.BLING_AJAX_TEMPLATES not in st.objectIds():
        addDirectoryViews(st, config.SKINS_DIR, config.GLOBALS)

    ajax = st.getSkinByName(config.AJAX_SKIN)
    if ajax: manage((config.AJAX_SKIN,), del_skin=1)

    for skinName in st.getSkinSelections():
        path = st.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        try:
            if config.BLING_SKIN not in path:
                path.insert(path.index('custom') +1, config.BLING_SKIN)
        except ValueError:
            if config.BLING_SKIN not in path:
                path.append(config.BLING_SKIN)

        path = ','.join(path)
        st.addSkinSelection(skinName, path)

    # Create the special layer by copying default
    default = st.getDefaultSkin()
    default_path = [i.strip() for i in
                    st.getSkinPath(default).split(',')]
    default_path.insert(default_path.index('custom') +1,
                        config.BLING_AJAX_TEMPLATES)
    default_path = ','.join([layer.strip() for layer in default_path])
    st._getSelections()[config.AJAX_SKIN] = default_path

    print >>out, "Customized skinpath for Bling"

def install(self):
    out = StringIO()
    update_skins(self, out)
    update_registries(self, out)

    print >> out, "Successfully installed %s." % config.PROJECT_NAME
    return out.getvalue()

