##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of Bling.
#
# Bling is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Bling is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Bling; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
'''
Bling
------
Ajax support utilities. These methods are called in the ZPTs

tal:define="ajax python:modules['Bling.ajax']"

tal:content="structure python:ajax.linkTo(url, target_id)"


$Id: $
'''
from StringIO import StringIO
import re

try:
    import json
    JSON_SUPPORT = True
except ImportError:
    JSON_SUPPORT = False


CALLBACKS = ['uninitialized', 'loading', 'loaded', 'interactive', 'complete', 'failure' , 'success']
CALLBACKS.extend([str(i) for i in range(100, 599)])

AJAX_OPTIONS = ["before", "after", "condition", "url", "asynchronous",
                "method", "insertion", "position", "form", "with", "update", "script"
                ] + CALLBACKS

def mapInsert(where):
    return {'before' : 'Insertion.Before',
            'after'  : 'Insertion.After',
            'top'    : 'Insertion.Top',
            'bottom' : 'Insertion.Bottom',
            }.get(where, "null")

def collapse(function):
    return function.replace("\n", " ")

def escape_js(js):
    if not js: return ''
    js = re.sub("\r\n|\n|\r", " ", js)
    js = re.sub("(['\"])", r"\\\1", js)
    return js

def javascript_tag(*content):
    return '''<script type="text/javascript"><!--
    %s
    --></script>''' % ";".join(content)


def build_callbacks(options):
    callbacks = {}
    for callback, code in options.iteritems():
        if callback in CALLBACKS:
            name = "on"+ callback.title()
            callbacks[name] = "function(request){%s}" % code
    return callbacks


def options_for_javascript(options):
    """returns the javascript text for a set of options"""
    o = []
    for k,v in options.items():
        if isinstance(v, bool):
            v = v and 'true' or 'false'
        elif isinstance(v, basestring) and not k.endswith("_literal"):
            # Gross, but simple for the developer
            # I could say if it contains ()|; as well
            if not v.startswith("$") and \
               not v.startswith("function") and \
               not k == "insertion" and \
               not v == "null":
                v = repr(v)

        o.append(" %s: %s" %(k,v))
    return "{%s}" %  ",".join(o)

def options_for_ajax(options):
    # We pass the options as a dict so we can modify it inplace
    # XXX: this needs to be extensible if I want to handle it like
    # this
    ajax = build_callbacks(options)

    ajax['method'] = options.get("method", "post")
    ajax['insertion'] = mapInsert(options.get('where'))
    ajax['asynchronous'] = options.get("synchronous", True)
    
    if options.get('form'):
        ajax['parameters'] = "Form.serialize(this)"
    elif options.get('submit'):
        ajax['parameters'] = "Form.serialize($('%s'))" % options['submit']
    elif options.get('with'):
        ajax['parameters'] = options['with']

    if 'js' in options: ajax.update(options['js'])
    return options_for_javascript(ajax)


def remoteFunction(url, **kwargs):
    update = ''
    function = StringIO()

    d = kwargs.get('update')
    if d:
        update = []
        if isinstance(d, dict):
            s = d.get('success')
            f = d.get('failure')
            if s:
                if not s.startswith("$"): s = "$('%s')" % s
                update.append("success: %s" % s)
            if f:
                if not f.startswith("$"): f = "$('%s')" % f
                update.append("failure: %s" % f)
            update = "{%s}" % ",".join(update)
        else:
            update = "$('%s')" % d

    # Function Style
    # is this a remote invocation or does it update
    # a known DOM target?
    if update:
        print >>function, "new Ajax.Updater(%s," % update
    else:
        print >>function, "new Ajax.Request(",

    # Server URL (which must be adjusted to the server env already)
    print >>function, "%r," % url

    # Javascript options
    print >>function,  options_for_ajax(kwargs)

    print >>function, ")"

    function = function.getvalue()
    if kwargs.get("before"):
        function = "%s;%s" %(kwargs['before'], function)
    if kwargs.get("after"):
        function = "%s;%s" %(function, kwargs['after'], )
    if kwargs.get("condition"):
        function = "if (%s) {%s}" %(kwargs['condition'], function)
    if kwargs.get("confirm"):
        function = "if (confirm('%s')) { %s }" % (
            escape_js(kwargs['confirm']),
            function)
    return function

def periodicalCall(code, frequency=10):
    return "new PeriodicalExecuter(function(){%s}, %s)" %(code, frequency)

def periodicalCallRemote(url, options=None, **kwargs):
    if options: kwargs.update(options)
    frequency = kwargs.get('frequency', 10)
    code = "new PeriodicalExecuter(function(){%s}, %s)" % (
        remoteFunction(url, **kwargs),
        frequency)
    return code

def linkTo(name, url, options = None, **kwargs):
    # register the js function
    if options: kwargs.update(options)
    function = remoteFunction(url, **kwargs)
    function = collapse(function)
    return '<a href="#" onclick="%s;return false">%s</a>' % (
        function,
        name)


def remoteForm(url, options=None, **kwargs):
    '''link a form to a remote url
    usage: tal:define="form python:ajax.remoteForm(url, options)"
           tal:attributes="onsubmit form/onsubmit;
                           method  form/method;
                           action  form/action; "
     '''
    if options: kwargs.update(options)
    function = remoteFunction(url, **kwargs)
    function = "%s;return false;" % collapse(function)
    return {'method' : kwargs.get("method", "post"),
            'action' : url,
            'onsubmit': function,
            }


def submitRemote(url, options=None, **kwargs):
    '''used in an input tag to create a ajax POST trigger
    usage: tal:define="form python:ajax.remoteForm(url, options)"
           tal:attributes="onclick form/onclick;
                           method  form/method;
                           action  form/action;
                           type    form/type"
    '''

    if options: kwargs.update(options)
    function = remoteFunction(url, **kwargs)
    function = "%s;return false;" % collapse(function)
    return {'method' : kwargs.get("method", "post"),
            'action' : url,
            'onclick': function,
            'type'   : 'button'}


def build_observer(klass, url, field_id, options):
    output = StringIO()
    if "with" not in options:
        # use the observed value by default
        if "withvalue" in options:
            options['with'] = "'name=%s&value=' + value" % (field_id)
        else:
            options['with'] = "%s('%s')" % (options.pop('serialize',
                                                        'Form.Element.serialize'),
                                            field_id)
        field_id = "$('%s')" % field_id


    callback = remoteFunction(url, **options)
    callback = collapse(callback)

    print >>output, "new %s(%s," % (klass, field_id)
    if options.get("frequency"):
        print >>output, "%s," % options['frequency']
    print >>output, "function(element, value) { %s })" % callback
    return output.getvalue()


def observeField(url, field_id=None, options=None, **kwargs):
    if options: kwargs.update(options)
    if kwargs.get("frequency"): klass = "Form.Element.Observer"
    elif kwargs.get("delay"): klass = "Form.Element.DelayedObserver"
    else: klass = "Form.Element.EventObserver"
    kwargs['serialize'] = 'Form.Element.serialize'
    return build_observer(klass, url, field_id, kwargs)

def observeForm(url, form_id, options=None, **kwargs):
    if options: kwargs.update(options)
    if kwargs.get("frequency"): klass = "Form.Observer"
    else: klass = "Form.EventObserver"
    kwargs['serialize'] = 'Form.serialize'
    return build_observer(klass, url, form_id, kwargs)



def eval_remote():
    return "eval(request.responseText)"


def update_element(element_id, options=None, **kwargs):
    if options: kwargs.update(options)
    content = kwargs.get("content", "")
    position = mapInsert(kwargs.get('where'))
    function = {
        'update' : "new Insertion.%s('%s', '%s')" % ( position,
                                                      element_id,
                                                      content),
        'empty'  : "$('%s').innerHTML = ''" % element_id,
        'remove' : "Element.remove('%s')" % element_id,
        }.get(kwargs.get('action'))

    return function

def visualEffect(name, target, options=None, **kwargs):
    if options: kwargs.update(options)
    return "new Effect.%s($('%s'),%s)" %(name, target, options_for_javascript(kwargs))


# Higher level API
def refreshArea(url, container, options=None, **kwargs):
    """ Canned behavior for an area that replaces itself according to
    some rule
    """
    if options: kwargs.update(options)
    visuals = kwargs.pop('visuals', None)
    if 'success' not in kwargs:
        kwargs['success'] = visualEffect("Highlight",
                                         container,
                                         visuals)
    options = options_for_ajax(kwargs)
    return "new Ajax.PeriodicalUpdater('%s', '%s', %s);" % (
                          container, url, options)


def checkedField(url, field_id, options=None, **kwargs):
    """ A canned behaviour. Check a field and manage its error state. """
    if options: kwargs.update(options)
    # The id of the error location
    error = kwargs.get("error", "error")
    # XXX: we encode parent Node which is useful in AT/Plone forms,
    # but in general might come off wrong, fix it
    onError = """%s; Element.addClassName($('%s'), 'error');Field.activate('%s');""" % (
        visualEffect('Appear', error),
        error,
        field_id)
    onSuccess = """Element.removeClassName($('%s'), 'error');%s;""" %(
        error, visualEffect('DropOut', error))

    return javascript_tag(observeField(url, field_id,
                        {'update' :
                         {  'success' : error,
                            'failure' : error},
                         'success' : onSuccess,
                         'failure' : onError}, **kwargs))

def BlingEditor(url, field_id, options=None, **kwargs):
    """will call url with <field_id>=value. Save and return the value
    in your method.

    takes rows, cols
    """

    if options: kwargs.update(options)
    options = options_for_ajax(kwargs)
    return javascript_tag("new Ajax.BlingEditor('%s', '%s', %s);" % (
        url, field_id, options))


def BlingLiveSearch(url, source, result):
    afterUpdateElement = """function(f, v) {
      target = v.getElementsByTagName('a').item(0).href;
      window.location=target;
      }"""
    return """new Ajax.Autocompleter('%s', '%s', '%s',
    {afterUpdateElement : %s});""" % (
        source, result, url, afterUpdateElement)

def inplaceCollectionEditor(url, field_id, collection=None,
                            options=None, **kwargs):

    if collection is None:
        collection = []
    if options: kwargs.update(options)
    kwargs['collection'] = collection
    options = options_for_ajax(kwargs)

    return "new Ajax.InPlaceColletionEditor('%s', '%s', %s);" % (
        field_id, url, options)


def rollover(target, orig, over):
    return """ new Effect.Rollover('%s', '%s', '%s'); """ %(target, orig, over)


def slider(handle, tracker, options=None, **kwargs):
    """Create a slider, the resultant output can be bound to a local
    form variable and/or sent directly to a target method.

    url='http://' || field_id='foo'

    to control behaviour
    """
    if options:kwargs.update(options)
    value = kwargs.get('value', 0)
    if 'onChange' not in kwargs:
        # is this a local form id
        # or a remote url?
        if 'url' in kwargs:
            kwargs['onChange'] = remoteFunction(kwargs['url'])
        else:
            oc = """function(v){$('%s').value = v;}""" %  (
                kwargs.get('field_id'),
                )
            kwargs['onChange'] = oc

    if 'values' in kwargs:
        values = kwargs['values']
        if 'range' not in kwargs:
            kwargs['range'] = '$R(%s, %s)' %(min(values), max(values))

    if 'display' in kwargs:
        display = kwargs['display']
        if not 'onSlide' in kwargs:
            slide = """function(v, slider) {var e = $('%s');
            e.innerHTML = v;slider.track.value = v;}""" % (display)
            kwargs['onSlide'] = slide

    for key in ['field_id', 'url', 'display', 'value']:
        if key in kwargs: del kwargs[key]

    options = options_for_javascript(kwargs)
    return "new Control.Slider('%s', '%s', %s);" % (
        handle, tracker, options)

_element_re = re.compile('''[\"\']?\$element[\"\']?''', re.M)
def Behavior(selector, js):
    """Register a behavior via a css selector, because element is
    always available we take explicit measures to make sure it can
    be used by other methods. To support this we look for $element
    and replace it in the JS with a direct reference to the element
    the behavior triggered on.
    """
    method = 'Behavior.register("%s", function(element) {%s});' %(selector,
                                                         js)

    # now subst element
    method = _element_re.sub('element', method)
    return javascript_tag(method)

def LinkReplacer(container, element="$element", options = None,
                 **kwargs):
    """take an existing link and convert it to make an AJAX call
    updating container. Most useful with Behavior and a selector
    """
    if options:kwargs.update(options)
    options = options_for_javascript(kwargs)
    return '''new Ajax.LinkReplacer('%s', "%s", %s});''' % (
        container, element, options)

def AutoCompleter(element, url, options=None, **kwargs):
    if options:kwargs.update(options)
    options = options_for_javascript(kwargs)
    return 'new PloneEngine.AutoCompleter("%s", "%s", %s)' %(element, url, options)



# UTILS
def list2UL(lst):
    """Convert a Python list to an unordered list suitable for use
    with Scriptaculous autocompleters
    """
    s = ["<ul>"]
    for el in lst:
        if el: s.append("<li>%s</li>" % el)
    s.append("</ul>")
    return '\n'.join(s)

def tokenize(value, tokens="(?m)[,\n]\s*"):
    """tokenize a stream"""
    return re.split(tokens, value)

def setStatus(response, msg):
    "provide a simple status message to the client"
    response.setHeader('X-JSON', 'PloneEngine.setStatus("%s")' % (msg,));


if JSON_SUPPORT:
    #
    # JSON support
    #
    def jsonify(orig):
        """A function decorator designed to marshal the function result as
        JSON. This is intended to ease client side scripting that is data
        rather then HTML dependent. This is not the primary target or
        usage of Bling, but the support is here
        """
        def f(*args, **kwargs):
            result = orig(*args, **kwargs)
            return json.write(result)
        return f


try:
    from StringIO import StringIO
    from Products.PageTemplates.Expressions import getEngine
    from TAL.TALInterpreter import TALInterpreter
    from Products.CMFCore.Expression import createExprContext
    from Products.CMFPlone.utils import IndexIterator

    engine = getEngine()
    def setupContext(obj, template, **kwargs):
        # create a new context object
        folder = obj.aq_parent
        utool  = obj.portal_url
        portal = utool.getPortalObject()

        context = createExprContext(folder, portal, obj)

        context.setLocal('here', obj)
        context.setLocal('context', obj)
        context.setLocal("utool", utool)
        context.setLocal('nothing', None)
        context.setLocal("portal", portal)
        context.setLocal("errors", {})
        context.setLocal("tabindex", IndexIterator(pos=30000))
        context.setLocal("template", template)
        return context

    def renderMacro(context, template, macro):
        ctx=setupContext(context, template)
        buffer = StringIO()
        # evaluate global_defines (to load all the vars we depend on)
        defines = context.unrestrictedTraverse("global_defines").macros['defines']
        TALInterpreter(defines, {}, ctx, buffer)()
        # Now eval the macro
        macro = context.unrestrictedTraverse(template).macros[macro]
        TALInterpreter(macro, {}, ctx, buffer)()
        return buffer.getvalue()

except ImportError:
    pass


__all__ = ['remoteFunction', 'linkTo', "remoteForm", "submitRemote",
           "observeField", 'visualEffect', 'rollover'
           ]

