// (C) Copyright 2006 ObjectRealms, LLC
// All Rights Reserved
//
// This file is part of Bling.
//
// Bling is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Bling is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Bling; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/*
  Singleton that manage status messaging, and some of the load/save primitives
*/
var PloneEngine = {
    loadingMsg : "loading...", 
    connectionFailure : 'Connection To Server Lost, Please try again in a few moments',

    //highlight : '#dee7ec',
    highlight : '#ffff99',

    // Customized for Bling! and Archetypes with Plone	    
    // Singleton for the live editor, we only want one at a time
    blingLiveEdit : false,
    
    // Count of autocompleters running on page
    autocompleter_index: 0,
    autocompletes : {},
    
    // Tab panels
    panels : {},

    setStatus: function(message){
	   if($('portal-status') == null){
	      var body = cssQuery('body')[0];
	      var div = Builder.node('div', {id : 'portal-status'});
	      var msg = Builder.node('span', {id : 'portal-status-message'});
	      var img = Builder.node('img', {src : 'logoIcon.gif', id : 'spinner', style : 'padding-right: 1em;'});
	      div.appendChild(img);
	      div.appendChild(msg);
	      body.appendChild(div);
	   }
	   var node = $('portal-status');
	   var msg = $('portal-status-message');
	   Element.update(msg, message);
	   new Effect.Appear(node);
    },
   
   hideStatus: function(){
	  new Effect.Fade($('portal-status'));
   },
   
   callInProgress : function(xmlhttp) {
	  switch (xmlhttp.readyState) {
	  case 1: case 2: case 3:
	     return true;
	     break;
	     // Case 4 and 0
	  default:
	     return false;
	     break;
	  }
   },

   handleTarget: function(url, options) {
	  // handle the uniform modification of a url (to a method/view/etc)
	  // with optional information that might impact how the target
	  // is modified before/after the call. For example
	  // url, {macro : 'portlet'} might be used to request just the
	  // portlet macro of a given view.
	  // passing 'context' can be used to rewrite the calling
	  // context 
	  // XXX: todo
   }
        
};

//Sortable with Plone friendly serialize
PloneEngine.Sortable = {
    serialize: function(element) {
	element = $(element);
	options = arguments[1] || {};
	return $(Sortable.findElements(element, options) || []).collect( function(item) {
		return (encodeURIComponent(options.name) + ":list=" + 
			encodeURIComponent(item.id));
	    }).join("&");
    }
};

var Path = {
    validate : function(str) {
	if (!str) str = document.location.pathname;
	return str;
    },
    split : function(str) {
	str = this.validate(str);
	return $A(str.split('/'));
    },
    
    join : function(ary) {
	return ary.join('/');
    },
    
    pop : function(str) {
	str = this.validate(str);
	ary = this.split(str);
	ary.pop();
	return this.join(ary);
    },
    
    push : function(str, next) {
	str = this.validate(str);
	ary = this.split(str);
	ary.push(next);
	return this.join(ary);
    }
};

// Replace exiting links (element) with an AJAX updater
// Pre/post call methods are easily bound through options
Ajax.LinkReplacer = Class.create();
Object.extend(Ajax.LinkReplacer.prototype, {
	initialize : function(container, el, options) {
	    this.container = $(container);
	    this.element = el;
	    this.options = Object.extend({ 
		    onSuccess : this.onSuccess,
		    onFailure : this.onFailure,
		    evalScripts: true,
		    method : 'get' }, 
		options || {});
	    
	    if(el.onclick)
		return;
	    this.target = el.href;
	    el.onclick = this.onClick.bindAsEventListener(this);
	},
	    
	    onClick : function(event){
		var targ;
		if (!event) var event = window.event;
		if (event.target) targ = event.target;
		else if (event.srcElement) targ = event.srcElement;
		if (targ.nodeType == 3) // defeat Safari bug
		    targ = targ.parentNode;
		
		if (this.options.onCreate) {
		    this.options.onCreate();
		}
		new Ajax.Updater(this.container,
				 this.target,
				 this.options
				 );
		return false;
	},
	    
	    onSuccess : function() {
	    if (this.options.onSuccess) {
		this.options.onSuccess();
	    }
	},
	    
	    onFailure : function() {
	    if (this.options.onFailure) {
		this.options.onFailure();
	    }
	}
	    
	    
	    });



Page = Class.create();
Page.prototype = {
    initialize: function(page, url, params) {
	this.page = page;
	this.url = url;
	this.params = params;
    }
}

var PageCache = {
    pages : {},
    load : function(url, params) {
	// we only want the path part of the URL, not a query string
	url = url.split(/\?/)[0];
	page = this.pages[url] || {};
	page = page[params];
	return page;
    },
    
    store : function(url, params, page) {
	url = url.split(/\?/)[0];
	page = new Page(page, url, params);
	thisUrl = this.pages[url] = this.pages[url] || {};
	thisUrl[params] = page;	
    }
};

// Register global responders that will occur on all AJAX requests
// This can handle timeouts as well as activating a spinner in the plone skin
Ajax.Responders.register({
	onCreate: function(request) {
	    request['timeoutId'] = window.setTimeout(
						     function() {
							 // If we have hit the timeout and the AJAX request is active, abort it and let the user know
							 if (PloneEngine.callInProgress(request.transport)) {
							     request.transport.abort();
							     PloneEngine.setStatus(PloneEngine.connectionFailure);
							     // Run the onFailure method if we set one up when creating the AJAX object
							     if (request.options['onFailure']) {
								 request.options['onFailure'](request.transport, request.json);
							     }
							 }
						     },
						     5000 // Five seconds
						     );
	    msg = request.options['status'] || PloneEngine.loadingMsg;
	    PloneEngine.setStatus(msg);
	},
	    
	    onException: function(request, exception) {
	    window.clearTimeout(request['timeoutId']);
	},
	
	    onComplete: function(request) {
	    window.clearTimeout(request['timeoutId']);
	    PloneEngine.hideStatus();
	    Behavior.apply();

	}
    });


var Cookie = {
    get: function( name ) {
	var start = document.cookie.indexOf( name + "=" );
	var len = start + name.length + 1;
	if ( ( !start ) && ( name != document.cookie.substring( 0, name.length ) ) ) {
	    return null;
	}
	if ( start == -1 ) return null;
	var end = document.cookie.indexOf( ";", len );
	if ( end == -1 ) end = document.cookie.length;
	return JSON.parse( unescape(document.cookie.substring( len, end ) ));
    },

    set: function ( name, value, expires, path, domain, secure ) {
	var today = new Date();
	today.setTime( today.getTime() );
	if ( expires ) {
	    expires = expires * 1000 * 60 * 60 * 24;
	}
	var expires_date = new Date( today.getTime() + (expires) );
	document.cookie = name+"="+ escape(JSON.stringify( value )) +
	( ( expires ) ? ";expires="+expires_date.toGMTString() : "" ) + //expires.toGMTString()
	( ( path ) ? ";path=" + path : "" ) +
	( ( domain ) ? ";domain=" + domain : "" ) +
	( ( secure ) ? ";secure" : "" );
    },

    del: function( name, path, domain ) {
	if ( getCookie( name ) ) document.cookie = name + "=" +
				     ( ( path ) ? ";path=" + path : "") +
				     ( ( domain ) ? ";domain=" + domain : "" ) +
				     ";expires=Thu, 01-Jan-1970 00:00:01 GMT";
    }
    
};


//Trival Rollover class
Effect.Rollover = Class.create();
Effect.Rollover.prototype = {
    initialize: function(element, orig, over) {
	    this.element = $(element);
	    this.default_url = orig;
	    this.over_url    = over;

	    this.mouseoverListener = this.enterHover.bindAsEventListener(this);
	    this.mouseoutListener = this.leaveHover.bindAsEventListener(this);
	    Event.observe(this.element, 'mouseover', this.mouseoverListener);
	    Event.observe(this.element, 'mouseout', this.mouseoutListener);
    },
    enterHover: function() { this.element.src = this.over_url;},
    leaveHover: function() { this.element.src = this.default_url;}
};

// Plone AutoCompleter
PloneEngine.AutoCompleter = Class.create();
PloneEngine.AutoCompleter.prototype = {
   initialize: function(element, url, options) {
      element = $(element);
	  options =  Object.extend({ 
		                          paramName: 'value',
		                          afterUpdateElement : function () {new Effect.Highlight(element);}
	                           }, options || {});
	  
	  if (PloneEngine.autocompletes[element.id]) {
	     choice = PloneEngine.autocompletes[element.id];
	  } else {
	     choice = 'autocomplete_choices_' + PloneEngine.autocompleter_index;
	     PloneEngine.autocompleter_index++;
	     
	     div = Builder.node('div', {
		                       className :'autocomplete', 
		                       id : choice});
	     element.parentNode.appendChild(div);
	     PloneEngine.autocompletes[element.id] = div;
	  }
	  new Ajax.Autocompleter(element, 
			                 $(choice), 
			                 url, options
			                );
   }
   
};


PloneEngine.Tab = Class.create();
PloneEngine.Tab.prototype = {
    initialize : function(panel, target, content, options) {
       
	this.panel = panel;
	this.target = target;
	this.content = content;
	this.loaded  = false;
	this.loading = false;
	this.active = false;

	// each tab gets its own content DIV
	
	this.container = this.panel.content.cloneNode(false);

	if (content)
	    Element.update(this.container, content);
	
	this.options = Object.extend({
		autoload : false
	    }, 
	    options || {});
	
	if (this.options.autoload) {
	    this.loadTab();
	}
	
	// Bind the handlers
	this.target.onclick  = this.showTab.bindAsEventListener(this);
    },

    loadTab : function() {
	if (this.loading) return;
	this.loading = true;
	new Ajax.Updater(this.container, this.target.href, {
		parameters : 'plone_skin=ajax',
		method : "get",
		onComplete: function(request) {
		    this.loading = false;
		    this.loaded = true;
		    // If a show triggered this we will be the active tab
		    // waiting for the load event
		    if (this.active) { this.updateTab(); }
		    (this.options.onLoad || Prototype.emptyFunction).apply(this);
		}.bind(this)
	    });
    },
    
    updateTab : function() {
	Element.removeClassName(this.target.parentNode, 'plain');
	Element.addClassName(this.target.parentNode, 'selected');
	//Element.replace(this.panel.content, this.container.innerHTML);
	this.panel.content.parentNode.replaceChild(this.container, this.panel.content);
	this.panel.content = this.container;
	Behavior.apply();
	(this.options.onShow || Prototype.emptyFunction).apply(this);

    },

    showTab : function() {
	// if the tab isn't loaded, load it
	this.panel.showTab(this);
	this.active = true;
	if (!this.loaded) {
	    if (this.loading)
		return false;
	    this.loadTab();
	} else 
	    this.updateTab();
	return false;
    },
    
    hideTab : function() {
	Element.removeClassName(this.target.parentNode, 'selected');
	Element.addClassName(this.target.parentNode, 'plain');
	
	// Store the DOM node
	this.container = this.panel.content

	if (this.active) {
	    this.active = false;
	    (this.options.onHide || Prototype.emptyFunction).apply(this);
	}
    }
};


PloneEngine.TabPanel = Class.create();
PloneEngine.TabPanel.prototype = {
    initialize : function (panel, content, options) {
	this.panel = $(panel);
	if (PloneEngine.panels[this.panel.id]){return;}
	PloneEngine.panels[this.panel.id] = this;

	this.content = $(content);
	this.options = Object.extend({
		selector : 'a',
		tabOptions : {}
	    }, 
	    options || {});
	
	this.tabs = new Array();
	
	this.findTabs().each(this.convertTab.bind(this));
	
    },
    
    findTabs : function () {
	return cssQuery(this.options.selector, this.panel);
    },
    
    convertTab : function(tab) {
	// Take a reference to a tab handle (ex, the link)
	// and re-write it to load/store/hide content in this.content
	// from async sources.
	target = tab.href;
	content = '';

	if (target == (this.options.cannonical || document.location)) {
	    content = this.content.innerHTML;
	}
	item = new PloneEngine.Tab(this, tab, content, this.options.tabOptions);
	this.tabs.push(item);
	
	if (target == (this.options.cannonical || document.location)) {
	    item.loaded = true;
	    item.active = true;
	    this.showTab(item);
	}
	
    },

    showTab : function(tab) {
	this.tabs.each(function (t) {
		if (t.active) t.hideTab();
	    });
    }

};

// AJAX in-place editor

Ajax.BlingEditor = Class.create();
Ajax.BlingEditor.prototype =   {
    initialize: function(url, element, options) {
	this.element = $(element);
	this.options = Object.extend({
		autopopulate: true, // see onLoadedExternalText
		autosize: true,
		min_width : "15em",
		toEditText : "Double Click the text to edit",
		savingText : "Saving...",
		onComplete: function(transport, element) {
		    Element.setOpacity(this.element, 0.0); 
		    Effect.Appear(this.element, { duration: 2.0 });
		},
		onFailure: function(transport) {
		    alert("Error communicating with the server: " + transport.responseText.stripTags());
		},
		callback: function(form) {
		    return Form.serialize(form);
		},
		submitOnBlur:true,
		ajaxOptions: {}
	    }, options || {});
	
	// Guess/Gather the base url and the urls that will
	// generate the input forms and handle submission
	this.url = url;
	
	if (this.options.editor) {this.editor = this.options.editor;
	} else { this.editor = this.url + "/ajax_widget"; }
	
	if (this.options.handler) {this.handler = this.options.handler;
	} else { this.handler = this.url + "/ajax_set"; }
	
	
	this.originalBackground = Element.getStyle(this.element, 
						   'background-color');
	if (!this.originalBackground) {
	    this.originalBackground = "transparent";
	}
	
	this.element.title = this.options.toEditText;
	
	this.onclickListener = this.enterEditMode.bindAsEventListener(this);
	Event.observe(this.element, 'dblclick', this.onclickListener);

	this.mouseoverListener = this.enterHover.bindAsEventListener(this);
	this.mouseoutListener = this.leaveHover.bindAsEventListener(this);
	Event.observe(this.element, 'mouseover', this.mouseoverListener);
	Event.observe(this.element, 'mouseout', this.mouseoutListener);

	// early visual indicator that element is editable
	new Effect.Highlight(this.element, {duration : 1.5, startcolor: PloneEngine.highlight});
    },

    enterHover: function() { 
	if (PloneEngine.blingLiveEdit) return;

	var height = Element.getHeight(this.element);
	var offs = Position.positionedOffset(this.element);
	var img = Builder.node('img', {src : 'edit.gif'});
	// once we properly load the img offset will be
	// Element.getDimensions(img).width
	var offset = 16;
	
	d = Builder.node('div', { id : 'inplaceEditor', 
				  style : 'position: absolute;' +	
				  'top: ' + offs[1] +'px;'+		
				  'margin: auto auto auto -' + offset + 'px;'+ 
				  'z-index: 5;' +			
				  'opacity: 0.999999; display: none;'
	    },
	    [ Builder.node('a', {}, [ img ] ) ]
	    )
	
	this.element.parentNode.insertBefore(d, this.element);
	new Effect.Appear(d);
    },
    leaveHover: function() { 
	var ipe = $('inplaceEditor');
	if (ipe)
	    Element.remove(ipe);
	
    },
    
    enterEditMode: function(evt) {
	if (this.saving) return;
	if (this.editing) return;
	if (PloneEngine.blingLiveEdit) return;
	this.editing = true;
	PloneEngine.blingLiveEdit = true;
	this.oldInnerHTML = this.element.innerHTML;
	this.onEnterEditMode();
	
	this.loadExternalText();
	
	// stop the event to avoid a page refresh in Safari
	if (evt) {
	    Event.stop(evt);
	}
	return false;
    },
    
    triggerMCE : function (target) {
	tinyMCE.execCommand('mceAddControl', true, target);
	this.tinymce = target;
	tinyMCE.execCommand('mceFocus', true, 'mce_editor_0');
    },


    loadExternalText: function() {
	//load the editor widget
	Element.addClassName(this.form, this.options.loadingClassName);
	var params = '';
	if (this.options.form) {
	    // In the case of AT we might pass
	    // field_name : xxx, mode : 'edit'
	    params = $H(this.options.form).toQueryString();
	}

	new Ajax.Request(
			 this.editor,
			 Object.extend({
				 asynchronous: true,
				 parameters: params,
				 method: 'post',
				 onComplete: this.onLoadedExternalText.bind(this)
			    }, this.options.ajaxOptions || {})
			 );
    },

    onLoadedExternalText: function(transport) {
	Element.removeClassName(this.form, this.options.loadingClassName);
	var dim = Element.getDimensions(this.element);
	this.element.style.visibility = "";

	// We need to parse the return text and see if it contains a 
	// form element. If it doesn't we need to wrap it
	var text = transport.responseText;
	this.form = Builder.node("form", {id: this.element.id + "-inplaceeditor"});
	Element.update(this.form, text);
	this.element.parentNode.insertBefore(this.form, this.element);
	
	tag = Form.findFirstElement(this.form);
	if (tag) {
	    if (!tag.id) {
		tag.id = this.element.id + "-inplaceeditor-widget";
	    }

	    if (this.options.autopopulate) {
		// We've been asked to populate the widget from the text
		// existing on the screen, this is only good for text
		// values and only good when the External Loader returns a
		// single field, if either of these conditions are
		// violated this will not work as expected.
		if (tag.tagName.toLowerCase() == 'textarea') {
		    tag.innerHTML = this.oldInnerHTML;		    
		} else {
		    tag.value     = this.oldInnerHTML;
		}
	    }	
	    if (this.options.autosize) {
		Position.clone(this.element, tag);
		// For some input tags the width will be too small by default
		dim = Element.getDimensions(this.element);
		if (dim.width < this.options.min_width) { 
		    tag.style.width = this.options.min_width;
		}
	    }
	    
	    
	    if (tag.tagName.toLowerCase() == 'textarea') {
		this.triggerMCE(tag.id);
	    } 
	    // This binding doesn't work with tiny right now
	    // but its generally what we want
	    Event.observe(tag, "keypress", 
			  this.onKeyPress.bindAsEventListener(this));
	    
	}
	// We do this late so we can match the size (on autosize)
	// hidden elements have no effective size
	Element.hide(this.element);
	//Update/set the event handlers
	Form.focusFirstElement(this.form);

	this.form.onsubmit = this.onSubmit.bind(this);
	this.editField.onblur = this.onSubmit.bind(this);
	this.editField.onchange = this.onSubmit.bind(this);
	
    },
    
    onKeyPress: function(event) {
	switch(event.keyCode) {
 	case Event.KEY_TAB:
	         this.onSubmit();
 	         Event.stop(event);
	case Event.KEY_ESC:
	         this.leaveEditMode();
	         Event.stop(event);
	         return;
	}
	// Allow return in text areas, but consume it elsewhere
	if (event.keyCode == Event.KEY_RETURN) {
	    if (event.target.tagName.toLowerCase() != 'textarea') {
 	         this.onSubmit();
 	         Event.stop(event);
	    } else {
		// We want to exapand the height of the text area as needed
		// doesn't shrink though ... this is still just a toy
		var dim = Element.getDimensions(event.target);
		var value = Form.Element.getValue(event.target);
		event.target.style.height = (dim.height + 
					     (dim.height / value.split("\n").length)) + "px";
	    }
	}
    },

    
    onFailure: function(transport) {
	this.options.onFailure(transport);
	if (this.oldInnerHTML) {
	    this.element.innerHTML = this.oldInnerHTML;
	    this.oldInnerHTML = null;
	}
	return false;
    },
    
    onSubmit: function() {
	// onLoading resets these so we need to save them away for the Ajax call
	var form = this.form;
	var params = '';
	if (this.options.form) {
	    // In the case of AT we might pass
	    // field_name : xxx, mode : 'edit'
	    params = $H(this.options.form).toQueryString();
	}
	if (params) params += "&";
	params += this.options.callback(form);

	// do this first, sometimes the ajax call returns before we
	// get a chance to switch on Saving...  which means this will
	// actually switch on Saving... *after* we've left edit mode
	// causing Saving...  to be displayed indefinitely
	this.onLoading();
	
	new Ajax.Updater({ 
		success: this.element,
		    // don't update on failure (this could be an option)
		    failure: null
		    },
	    this.handler,
	    Object.extend({
		    method : 'post',
		    parameters: params,
		    onComplete: this.onComplete.bind(this),
		    onFailure: this.onFailure.bind(this)
		   }, this.options.ajaxOptions)
	    );
	// stop the event to avoid a page refresh in Safari
	if (arguments.length >  1) {
	    Event.stop(arguments[0]);
	}
	return false;
    },
    
    onLoading: function() {
	this.saving = true;
	this.removeForm();
	this.leaveHover();
	this.showSaving();
  },
    
    showSaving: function() {
	this.oldInnerHTML = this.element.innerHTML;
	this.element.innerHTML = this.options.savingText;
	Element.addClassName(this.element, this.options.savingClassName);
	this.element.style.backgroundColor = this.originalBackground;
	Element.show(this.element);
    },
    
    removeForm: function() {
	if(this.tinymce) {
	    tinyMCE.execCommand('mceRemoveControl', true, this.tinymce);
	    this.tinymce = false; 
	}
	if(this.form) {
	    if (this.form.parentNode) Element.remove(this.form);
	    this.form = null;
	}


    },
    
    leaveEditMode: function() {
	Element.removeClassName(this.element, this.options.savingClassName);
	this.removeForm();
	this.element.style.backgroundColor = this.originalBackground;
	Element.show(this.element);

	PloneEngine.blingLiveEdit = false;	
	this.editing = false;
	this.saving = false;
	this.oldInnerHTML = null;
	this.onLeaveEditMode();
    },
    
    onComplete: function(transport) {
	this.leaveEditMode();
	this.options.onComplete.bind(this)(transport, this.element);
  },

    onEnterEditMode: function() {},
    onLeaveEditMode: function() {},

    dispose: function() {
	if (this.oldInnerHTML) {
	    this.element.innerHTML = this.oldInnerHTML;
	}
	this.leaveEditMode();
	Event.stopObserving(this.element, 'dblclick', this.onclickListener);
    }
};


var BlingHistory = new Class.create();
BlingHistory.prototype = {
    initialize : function(container, url, variable) {
	// take the url and the value of variable 
	this.container = container;
	this.url = url;
	this.variable = $(variable);
	this.target = url + "/hist_rev/" + this.variable.value;
	new Ajax.Updater(this.container, 
			 this.target, {
			     method: 'get',
//  				 onComplete : function (request) {
//  				 diffString(this.container.innerHTML,
// 								   request.responseText);
// 			     }
 				 }
			 );
    }
};


if (false) {
Behavior.register("#content-panel",
		  function(el){
		      new PloneEngine.TabPanel(el,
					       "region-content", {
						   selector : "li a",  
						   tabOptions : {
						       onShow: function () {
							   new Effect.Appear(this.panel.content);
						       },
							   autoload: false
							   }
		      });
		  });							   
}

if (false) {
Behavior.register('.contentViews a',  function(el) {
		      new Ajax.LinkReplacer($('region-content'), el, {
			      cache : 1, 
			      onCreate : function() {
				  // remove selected class from all
				  cssQuery(".contentViews .selected").each(
									   function(tag) {
									       Element.removeClassName(tag, 'selected');}
									   );
				  Element.addClassName(el.parentNode, 'selected');
			      },
				  parameters : 'plone_skin=ajax'
			 });
    });
}


Behavior.register("#calendar-next a, #calendar-previous a",
		  function(el){
		      new Ajax.LinkReplacer($('thePloneCalendar').parentNode, el);
		  });

// Schemata/Fieldset handling

// FC ordering
if (false) {
  // this is in 2.5
Behavior.register('.orderable tbody', 
		  function (el) {
		      //Find and record all the orderable elements 
		      // in a hidden that we will include with serialize
		      orderables = el.id + "_orderables";
		      var cur = $(Sortable.findElements(el,  {tag : 'tr'}) || []).collect( function(item) {
			      return (encodeURIComponent(item.id));
			  }).join(",");
		      
		      var hidden = $(orderables);
		      if (!hidden) {
			  hidden = Builder.node('input', {
				  'type' : 'hidden',
				  id : orderables,
				  value : cur
			      });
			  el.parentNode.appendChild(hidden);
		      } else {
			  hidden.value = cur;
		      }

		      Sortable.create(el, {tag : 'tr', 
				  name : 'value',
				  constraint : 'vertical',
				  dropOnEmpty : true ,
				  onChange : function(container) {
				  //reorder even/odd in the container
				  Sortable.findElements(el, {tag : 'tr'}).each(function (row, index) {
					  style = (index % 2 == 0) && 'even' || 'odd';
					  Element.removeClassName(row, 'odd');
					  Element.removeClassName(row, 'even');
					  Element.addClassName(row, style);
					  Element.addClassName(row, "selected");
					  row.style.zindex = -1;
				      });
			      },
				  onUpdate : function(container) {
				  new Ajax.Request(Path.push(Path.pop(), "bling_order_folder"), 
						   {parameters : PloneEngine.Sortable.serialize(el, {tag : 'tr', name : 'value'}) +"&orig=" + $F(hidden), 				 
							   status : 'saving order ...',
							   evalScripts: true
							   });
			      }
			  });
		  });

}


if (false) {
// Remember the drag -- portlet positioning We would never really do
// this w/o speaking to a tool that kept property settings that
// enabled or disabled such behavior first.
Behavior.register(".portlet, .ploneCalendar",
		  function (el) {
		      Element.makePositioned(el);
		      Position.absolutize(el);
		      pos = Position.page(el);

		      function record(d) {
			  // this would really be a MemoryDraggable w/
			  // a relativeTo init handler that was the path
			  Cookie.set(el.id, Position.page(el), 1, "/");
		      }
		      function recall(el) {
			  c = Cookie.get(el.id);
			  if (c) {
			      c = eval(c);
			      c[0] = parseInt(c[0]);
			      c[1] = parseInt(c[1]);
			      //p = Position.realOffset(el);
			      el.style.left = (c[0]) + 'px';
			      el.style.top = (c[1]) + 'px';
			  }
		      }
		      new Draggable(el, {revert:false, change: record, ghosting: true});
		      recall(el);
		  });

}

if (window.tinyMCE) {
   tinyMCE.init({
	               theme : "advanced",
	               mode: "exact",
	               content_css : 'tinyContent.css',
	               plugins: "inlinepopups, save, cleanup",
	               extended_valid_elements : "a[name|href|target|title|onclick],img[class|src|border=0|alt|title|hspace|vspace|width|height|align|onmouseover|onmouseout|name],hr[class|width|size|noshade],span[class|align|style]",
	               theme_advanced_toolbar_location : "bottom",
	               theme_advanced_buttons1 :"save,bold,italic,underline,separator,bullist,numlist,separator,undo,redo",
	               theme_advanced_buttons2 : "",
	               theme_advanced_buttons3 : "",
	               theme_advanced_toolbar_location : "bottom",
	               theme_advanced_toolbar_align : "left",
	               theme_advanced_resizing : true,
	               theme_advanced_resizing_use_cookie : false,
	               theme_advanced_statusbar_location : "bottom"
	               //auto_resize: true
	            });
}


// JavaScript diff code thanks to John Resig (http://ejohn.org)
// http://ejohn.org/files/jsdiff.js
function diffString( o, n ) {
    var out = diff( o.split(/\s+/), n.split(/\s+/) );
    var str = "";

    for ( var i = 0; i < out.n.length - 1; i++ ) {
	if ( out.n[i].text == null ) {
	    if ( out.n[i].indexOf('"') == -1 && out.n[i].indexOf('<') == -1 && out.n[i].indexOf('=') == -1 )
		str += "<b style='background:#E6FFE6;' class='diff'> " + out.n[i] +"</b>";
	    else
		str += " " + out.n[i];
	} else {
	    var pre = "";
	    if ( out.n[i].text.indexOf('"') == -1 && out.n[i].text.indexOf('<') == -1 && out.n[i].text.indexOf('=') == -1 ) {

		var n = out.n[i].row + 1;
		while ( n < out.o.length && out.o[n].text == null ) {
		    if ( out.o[n].indexOf('"') == -1 && out.o[n].indexOf('<') == -1 && out.o[n].indexOf(':') == -1 && out.o[n].indexOf(';') == -1
			 && out.o[n].indexOf('=') == -1 )
			pre += " <s style='background:#FFE6E6;' class='diff'>" + out.o[n] +" </s>";
		    n++;
		}
	    }
	    str += " " + out.n[i].text + pre;
	}
    }

    return str;
}

function diff( o, n ) {
    var ns = new Array();
    var os = new Array();

    for ( var i = 0; i < n.length; i++ ) {
	if ( ns[ n[i] ] == null )
	    ns[ n[i] ] = { rows: new Array(), o: null };
	ns[ n[i] ].rows.push( i );
    }

    for ( var i = 0; i < o.length; i++ ) {
	if ( os[ o[i] ] == null )
	    os[ o[i] ] = { rows: new Array(), n: null };
	os[ o[i] ].rows.push( i );
    }

    for ( var i in ns ) {
	if ( ns[i].rows.length == 1 && typeof(os[i]) != "undefined" && os[i].rows.length == 1 ) {
	    n[ ns[i].rows[0] ] = { text: n[ ns[i].rows[0] ], row: os[i].rows[0] };
	    o[ os[i].rows[0] ] = { text: o[ os[i].rows[0] ], row: ns[i].rows[0] };
	}
    }

    for ( var i = 0; i < n.length - 1; i++ ) {
	if ( n[i].text != null && n[i+1].text == null && o[ n[i].row + 1 ].text == null &&
	     n[i+1] == o[ n[i].row + 1 ] ) {
	    n[i+1] = { text: n[i+1], row: n[i].row + 1 };
	    o[n[i].row+1] = { text: o[n[i].row+1], row: i + 1 };
	}
    }

    for ( var i = n.length - 1; i > 0; i-- ) {
	if ( n[i].text != null && n[i-1].text == null && o[ n[i].row - 1 ].text == null &&
	     n[i-1] == o[ n[i].row - 1 ] ) {
	    n[i-1] = { text: n[i-1], row: n[i].row - 1 };
	    o[n[i].row-1] = { text: o[n[i].row-1], row: i - 1 };
	}
    }

    return { o: o, n: n };
}

