//
// (C) Copyright 2006 ObjectRealms, LLC
// All Rights Reserved
//
// This file is part of Bling.
//
// Bling is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Bling is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Bling; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
function clamp(n, min, max) {return Math.min(Math.max(n, min), max);}

Node = Class.create();
Object.extend(Node.prototype, Enumerable);
Object.extend(Node.prototype, 
			  {
                 // This is a very simple DAG
                 // TreeItem uses this to manage a real tree
                 
				 // Tree navigation
				 root : function() { 
					if(this.options && this.options.root)
                       return this.options.root;
                    var obj = this;
					while (obj.parent) obj = obj.parent;
                    this.options.root = obj;
					return obj;
				 },

				 parentIndex : function() {
                    // Which element in your parent are you?
					sib = this.siblings()
					if (sib) return sib.indexOf(this);
					return -1;
				 },
				 
				 siblings : function() {
					if(this.parent) { return this.parent.children;}
					return $A();
				 },
				 
				 depth : function() {
					var d = 0, o = this;
					while (o.parent) { d++; o = o.parent; }; 
                    return d;
				 },
				 
				 _next : function() {
					var ob = this;
					var idx, c;
					
					if (this.children && 
                        this.children.length &&
                        this.options.expanded) 
                       return this.children[0]
					
					idx = this.parentIndex();
					c = this.siblings()[idx+1];
					if (c) return c;
					
					ob = this.parent;
					do {
					   idx = ob.parentIndex();
					   if (idx == ob.siblings().length) continue;
					   c = ob.siblings()[idx+1];
					   if (c) return c;
					} while (ob = ob.parent);
					return null;
				 },
				 
				 _prev : function() {
					// prev node including ancestors
					var idx = this.parentIndex();
					var ob = this.siblings()[idx-1];
					if (!ob) { 
					   if (this.parent) return this.parent; 
					   else return null;
					}
					while (ob && ob.children && ob.children.length) {
					   ob = ob.children.last()
					}
					return ob;
				 },
				 
				 next : function (filter) {
					var n = this;
					do {
					   n = n._next(); 
					   if (n && !filter || filter.bind(n)()) return n;
					} while(n);
					return null;
				 },
				 
				 prev : function (filter) {
					var n = this;
					do {
					   n = n._prev(); 
					   if (!filter || filter.bind(n)()) return n;
					} while(n);
					return null;
				 },
                 
                 addNode : function(node, position, offset) {
                    //position = position || -1;
                    offset = ((offset || 1) > 0) && 1 || -1;
                    var idx;

                    if (!this.children) {
					   this.children = $A();
					   this.addToggle();
					}

                    if (typeof position == 'number') {
                       idx = clamp(position, 0, this.children.length);
                    } else if (typeof position == 'object') {
                       // it should be another node
                       //XXX:  assert position.parent == this
                       idx = position.parentIndex();
                    } else {
                       idx = this.children.length;
                    }
                    
                    // now we have a position and an offset
                    // insert into this.children and this.spawn 
                    this.children.splice(idx, 0, node);
                    if (idx > this.children.length || !this.spawn.firstChild) {
                       this.spawn.appendChild(node.element);
                    } else {
                       this.spawn.insertBefore(node.element, this.spawn.childNodes[idx + offset]);
                    }

                    // its ok to set this again
                    // and its useful to do so in the replace/insert cases
                    node.parent = this;

                    //Make this nodes children a valid drop target
                    if (node.acquire('droppable') && node.entry) {
                       Droppables.add(node.entry, { 
                                         tree: true, 
                                         greedy: false,
                                         hoverclass : 'tree-drop',
                                         overlap : 'horizontal',
                                         onHover: function(drag, drop, over) {
                                            // Drag is ti.element and drop is ti.label
                                            // from these we need the real ti. 
                                            // we could do a search, but for now lets use an annotation. 
                                            // This is hooked up in parseSpec.
                                            drag = drag._node; drop = drop._node
                                            if (drag && drop && drag != drop) {
                                               this.dispatch('onHover', drag, drop, over);
                                            }
                                         }.bind(this),
                                         
                                         onDrop: function(drag, drop) {
                                            drag = drag._node; drop = drop._node;
                                            if (drag && drop && drag != drop) {
                                               this.dispatch('onDrop', drag, drop);
                                            }
                                         }.bind(this)
                                      }
                                     );
                    }
                    
                    this.readyGrid();
                    
					node.onAdd();
					return node;
                 },

				 remove : function() {
					//remove this item and its children from the tree
					var idx = this.parentIndex();
					var parent = this.parent;
					
                    // XXX: if this doesn't throw do the delete
                    this.onRemove();
                    
					parent.spawn.removeChild(this.element);
					parent.children = parent.children.reject(function(e) {return e && (e == this)}.bind(this));
                    this.parent = null;
                    
                    // XXX: from option
                    Droppables.remove(this.spawn);
				 },

                 reparent: function(node, position) {
                    // remove node from its current parent adding it under this node
                    if (node.parent) node.remove();
                    node.parent = this;
                    this.addNode(node, position);
                 },
                 
                 replace : function (node) {
                    // replace node with this
                    // the original node is returned w/o a parent 
                    var parent = node.parent;
                    var idx = node.parentIndex();

                    parent.children[idx] = this;
                    parent.spawn.replaceChild(this.element, node.element);
                    this.parent = parent;
                    node.parent = null;
                    
                    this.onAdd();
                    node.onRemove();
                    return node;
                 },

                 insertAbove : function(node) {
                    // insert _this_ element as a parent to node
                    this.addNode(this.replace(node));
                 },
                 
                 insertAfter : function(node, reference) {
                    if (node.parent) node.remove();
                    node.parent = this;
                    this.addNode(node, reference);
                 },
                 
                 ancestors : function() {
                    var root = this.root();
                    var ancestors = [];
                    var ob = this;
                    do {
                       ancestors.unshift(ob);
                       ob = ob.parent;
                    } while (ob != root);
                    return ancestors;
                 },

                 
                 walk : function(child) {
                    // return the child of this element matching name.
                    // this is taken from getToken so usually by id
                    return this.children.find(function (e) {
                                                 if (e.options && 
                                                     e.options.attrs && 
                                                     e.options.attrs.id == child) return e; 
                                                 return false;
                                              }
                                             );
                 },
                 
                 // Support Enumeration 
				 _each : function(iterator) {
					iterator(this);
					if (this.children) {
					   this.children._each(function(e) {e._each(iterator)});
					}
					
				 }

                 
			  });


TreeItem = Class.create();
Object.extend(TreeItem.prototype, Node.prototype);
Object.extend(TreeItem.prototype, {
				 initialize: function(specification, options, parent) {
					this.options = {
					   expanded : false,
					   selected : false,
					};
					this.parent = parent;

					this.parseOptions(options);
					this.parseSpecification(specification, this.options);
					
					this.bindListeners();
					this.onCreate();
				 },


				 bindListeners : function() {
					Event.observe(this.label, "keyup", this.onKeyPress.bindAsEventListener(this));
					Event.observe(this.label, "click", this.onClick.bindAsEventListener(this));
				 },

				 parseSpecification : function(specification, options) {
					// specification:
					// if the spec is an element we convert it
					// if its a string we populate a new item with it
					// arg[1]:
					// if array we add those items as children
					// arg[2]:
					// options as hash (unless there is no arg[1])

					if (typeof specification == 'string') {
					   //Element from string
					   specification = this.itemFromString(specification, options.attrs || {});
					} else {
					   if (specification.element)
						  specification = specification.element;
					}
					this.text = specification;

					// Convert Element to tree style markup
					// DIV 
					//    TABLE (label) w/1 row
					//     SPAN
					//       ..for each child..
					//         DIV
					//            TABLE w/1 row
					//            SPAN
					// 
					// Using this model we can take pretty much what ever HTML
					// or elements they might want to pass to us and feed it
					// into the table. Its only the elements we create that
					// need event bindings so we have full control over the
					// 
					var spec = $A();
					var icon = this.acquire('icon');
					if (icon) {
					   spec.push(Builder.node('img', {'src' : icon}));
					}
					spec.push(specification);
                    
					this.label = Builder.node('td', {id : this.root().generateId()}, 
                                              [Builder.node('span', spec)]);
					this.button = Builder.node('img', {
												  src : this.acquire('images.empty'),
												  align : 'right',
											   });
					this.entry = Builder.node('table', {cellpadding : '0', cellspacing : '0'},
							[ Builder.node('tbody',
							  [ Builder.node('tr', 
											 [ Builder.node('td', 
															[this.button]),
											 this.label
											 ]) 
							  ])
							]);
                    

                    this.spawn = Builder.node('div', {className : 'tree-item-container'}, 
                                              [ Builder.node('div', {style : "white-space: nowrap; margin-left : 16px;"})]);	
					this.element = Builder.node('div', {style : "white-space: nowrap; margin-left : 16px;"},
												[this.entry, this.spawn]
												);
                    
                    // Annotations
                    this.element._node = this;
                    this.label._node = this;
                    this.entry._node = this;
					return this.element;
				 },
				 
				 toString : function() { 
                    token = this.dispatch('getToken');
					return ('#[TreeItem: ('+ token + ')]'); },

				 itemFromString: function(str, options) {
					// This handles arbitrary HTML
					var b = Builder.node('span', Object.extend({className : 'tree-item', 
																tabindex : "-1"}, 
															   options || {}));
					Element.update(b, str);
					return b;
				 },
				 
				 parseOptions : function(options) {
                    // extend/modify options

                    // make it easy to pass in id
                    if ((options && options.id) && (!options.attrs || !options.attrs.id)){
                       if (!options.attrs) options.attrs = {};
                       options.attrs.id = options.id;
                    }
					Object.extend(this.options, options);
				 },

                 dispatchCallback : function(name) {
                    // attempt to look up a callback using the following search path:
                    // this.onName is called which calls this and then...
                    // this.options[onName]
                    // this.root().options[onName]
                    // we don't search up each tree level looking, but 
                    // an implementation of that should follow logically if 
                    // you need it 
                    // this['on' + name] || 
                    var methName = 'on' + name;
                    var method = (this.options[methName] ||
                                  this.root().options[methName] || 
                                  Prototype.emptyFunction).bind(this);
                    method();
                 },

                 dispatch : function (name) {
                    // directly dispatch a call looking for the most
                    // closely binding method to call
                    var method = (this[name] ||
                                  this.options[name] ||
                                  this.root().options[name] || 
                                  Prototype.emptyFunction).bind(this);
                    var args = $A(arguments) || [];
                    args.shift();
                    return method.apply(this, args);
                    
                 },
                 
                 acquire : function (prop) {
                    // find the most relevant property 
                    // in a search from this.options to root.options
                    var obj = this;
                    if (prop.indexOf('.') == -1)
                       obj = obj.options[prop] || this.root().options[prop];
                    else {
                       var parts = prop.split('.');
                       var p = parts.shift();
                       obj = obj.options[p] || this.root().options[p];
                       for (var i=0;i<parts.length;i++){
                          obj = obj[parts[i]];
                       }
                    }
                    return obj;
                 },
                 
                 createItem : function(item, options, klass) {
					var ti;
					// XXX: this.constructor
					klass = klass || TreeItem;

					ti = new klass(item, options, this);
                    return ti
                 },

				 addItem : function(item, options, klass) {
                    var ti = this.createItem(item, options, klass);
                    ti = this.addNode(ti);
                    if (ti.options.selected) ti.mark(); 
                    if (ti.options.expanded) ti.expand();

                    if (this.acquire('draggable')) 
                       new Draggable(ti.element, {revert : true, 
                                                  ghosting: true, });
                    return ti;
				 },

				 addToggle : function() {
					var images = this.root().options.images;
					this.button.src = this.options.expanded && images.close || images.open;
					this.button.onclick = this.toggle.bindAsEventListener(this);
				 },
				 
                 
                 cursor : function () {return this.root().cursor();},
				 setCursor : function() {this.root().setCursor(this);},

				 visible : function() {
					// To be visable all its parents
					// must be expanded.
					var ob = this;
					while (ob = ob.parent) {
					   if (ob == null) break;
					   if (!ob.options || !ob.options.expanded) return false;
					} 
					return true;
				 },
                 
                 // High level selection code
				 selectNext : function(filter) {
					var obj = this.cursor();
					n = obj.next(filter);
					if(!n) return;
					obj.unhighlight();
					n.setCursor();		   
					n.highlight();	    
					n.onCursor();
					obj.onDecursor();		    
				 },

				 selectPrev : function(filter) {
					var obj = this.cursor();
					n = obj.prev(filter);
					if (!n) return;
					obj.unhighlight();
					n.setCursor();
					n.highlight();	    
					n.onCursor();
					obj.onDecursor();		    
				 },
				 
				 focus : function() {this.text.focus();}, 
				 
				 collapse : function() {
					if (!this.children) return;
					this.options.expanded = false;
					this.button.src = this.root().options.images.open;
					this.onCollapse();
				 },

				 expand : function() {
					if (!this.children) return;
                    if (this.options.url && !this.children.length) {
                       new Ajax.Request(this.options.url,
										{ method: "get",
                                          parameters: $H(this.acquire('url_parameters') || {}).toQueryString(),
										  onSuccess : this.populate.bind(this)
                                        });
                       
                    }
					this.options.expanded = true;
					this.button.src = this.root().options.images.close;
					this.onExpand();

				 },
				 
                 expandTo : function() {
                    // Expand the parent hierarchy exposing this node
                    var ob = this;
                    var root = this.root();
                    while((ob = ob.parent) && ob != root) { ob.expand();} 
                    this.setCursor();
                 },
                 
                 readyGrid : function() {
                    if (!this.button || !this.acquire('grid')) return;
                    var img;
                    if (this.children) {
                       if (this.options.expanded) {
                          if (this == this.siblings().last())
                             img = this.acquire('images.closebottom');
                          else 
                             img = this.acquire('images.close');
                       } else {
                          if (this == this.siblings().last())
                             img = this.acquire('images.openbottom');
                          else 
                             img = this.acquire('images.open');
                       }
                       
                       this.children.each(function (e) {e.readyGrid()});

                    } else {
                       if (this == this.siblings().last()) 
                          img = this.acquire('images.joinbottom');
                       else 
                          img = this.acquire('images.join');
                    }
                    
                    this.button.src = img;
                    
                    if (this.depth() > 1 && 
                        this != this.siblings().last()) {
                       this.spawn.style['backgroundImage'] = 'url('+ this.acquire('images.line') +')';
                       this.spawn.style['backgroundRepeat'] = "repeat-y";
                    }
                 },

				 toggle : function() {
					if (this.options.expanded) this.collapse();
					else this.expand();					
					this.setCursor();
				 },
				 
				 _select : function() {
					if (!this.root().options.multiple) {
					   /* if we don't allow multiple selection
                        * deselect the current item.
                        */ 
					   var sel = this.root().selected(Prototype.K);
					   if (sel && sel.length) {
						  sel[0].unmark();
					   }
					}
				 },

				 mark : function() { 
					this._select();
					this.options.selected = true;
					Element.addClassName(this.label, "tree-selected");
					this.onSelect();
				 },
				 
				 unmark : function() { 
					this.options.selected = false;
					Element.removeClassName(this.label, "tree-selected");
					this.onDeselect();
				 },
				 
                 toggleSelected : function() {
                    if (!this.acquire('selectable')) return;
					if (this.options.selected) this.unmark();
					else this.mark();
					
				 },

				 highlight : function() {
					Element.addClassName(this.label, "tree-highlight");
				 },
				 
				 unhighlight : function() {
					Element.removeClassName(this.label, "tree-highlight");
				 },
				 
				 
				 // Event Handlers
				 onKeyPress: function(event) {
					switch(event.keyCode) {
					case 32: // space
					   this.toggleSelected();
					   break;
					case Event.KEY_LEFT:
					   if (this.options.expanded) this.collapse();
					   break;
					case Event.KEY_RIGHT:
					   if (!this.options.expanded) this.expand();
					   break;
					case Event.KEY_UP:
					   this.selectPrev(this.visible);
					   break;
					case Event.KEY_DOWN:
					   this.selectNext(this.visible);
					   break;
					}
					Event.stop(event);
				 },

				 // Callbacks
				 onCreate : function() {this.dispatchCallback('Create');},

				 onAdd : function() {this.dispatchCallback('Add');},
				 onRemove : function() {this.dispatchCallback('Remove');},

				 onSelect : function() {this.dispatchCallback('Select');},
				 onDeselect : function() {this.dispatchCallback('Deselect');},
				 
				 onCursor: function() {this.dispatchCallback('Cursor');},
				 onDecursor: function() {this.dispatchCallback('Decursor');},
				 
				 onExpand : function() {this.dispatchCallback('Expand');},
				 onCollapse : function() {this.dispatchCallback('Collapse');},

                 onClick : function() {this.dispatchCallback('Click');},
                 //JSON support 
				 populate: function(transport, object) {
                    // given an array of arguments to addItem in JSON format
                    // do the right thing.
					// XXX: sniff content for JSON/HTML/whatever and dispatch
					// for now just JSON
					var data = eval(transport.responseText);
					this.populateFromJSON(data);
				 },
				 
				 populateFromJSON : function(data) {
					// Walk the return data adding nodes
					data.each(function (d) {
                                 var options = d[1];
                                 var klass;
                                 if(options) {
                                    var klass = options['klass'];
                                    if (klass && typeof klass == "string") {
                                       // JSON name -> class object
                                       klass = eval(klass);
                                    }
                                 }
								 var n = this.addItem.bind(this)(d[0], d[1], klass);
                                 kids = d[2];
                                 if (kids) {
                                    n.populateFromJSON(kids);
                                 }
							  }.bind(this));
				 }

			  });

Tree = Class.create();
Object.extend(Tree.prototype, TreeItem.prototype);
Object.extend(Tree.prototype, {
				 initialize: function(options) {
					this.options = {
					   images : { 
						  open : "plus.gif",
                          openbottom : "plusbottom.gif",
                          
						  close : "minus.gif",
                          closebottom : "minusbottom.gif",

						  empty : "spacer.gif",
                          
                          // Tree grid
                          join : 'join.gif',
                          joinbottom : 'joinbottom.gif',
                          line : 'line.gif',
					   },
                       grid : true, // show the tree grid?
                       allow_children : true,
                       selectable: true, 
					   expanded : true, 
					   selected : false,
					   multiple: false, // allow multiselection?
                       droppable: true, // does the tree support drop targets
                                        // this can be turned on for individual nodes
                                        // as well.
                       draggable : true, // can the nodes be dragged for dropping?
                       // default callbacks for tree
                       onExpand : function () {this.spawn.show();},
                       onCollapse: function () {this.spawn.hide();},
                       onCreate:   function() {
                          if (this.options.url) {
                             this.options.expanded = false;
					         this.addToggle();
					         this.children = $A();
                          }
                       },

                       getToken : function() {
                       	  return this.options.uid || 
                             this.text && (this.text.id || this.text.innerHTML) ||
                             '';
                       },
                       
                       getPath : function () {
                          // return the path from the root to this node
                          return this.ancestors().collect(
                             function(e){return e.dispatch('getToken')}).join('/');
                       },
                       
                       onDrop : function (drag, drop) {
                          if (drop.acquire('allow_children')) {
                             drop.reparent(drag);
                          } else {
                             // We want to add it to the parent of drop, 
                             // after drop
                             drop.parent.insertAfter(drag, drop);
                          }
                       },

				       onClick : function() {
					      this.cursor().unhighlight();
					      this.toggleSelected();
					      this.setCursor();
				       },

                    };
					
					this.parent = null;
					this._cursor = this;
					this.spawn = Builder.node('div', {className : 'tree'});

					this.parseOptions(options);
					this.onCreate();
				 },
				 
				 root   : function() { return this; },
				 cursor : function() { return this._cursor;},
				 setCursor : function(element) { 
					if (!element) element = this;
					this._cursor = element;
					if (this._cursor) this._cursor.focus();
				 },
				 
				 addToggle : function() {},
				 
                 focus : function() {
					if (!this.children) return;
					this.children[0].highlight();
					this.setCursor(this.children[0]);
				 },
                 
				 render : function(anchor) {
					// Render this tree bound to the anchor
					anchor = $(anchor);
					if (anchor) {
					   var id = anchor.id;
					   anchor.parentNode.replaceChild(this.spawn, anchor);
					   this.spawn.id = id;
                    } else {
                       document.body.appendChild(this.spawn);
                    }
                    this.onRender();
				 },
				 
                 onRender : function() {
                    this.dispatchCallback('Render');
                 },
                 
                 //Tree manipulation
				 selected : function(iterator) {
                    iterator = iterator || function(e) { return e.dispatch('getPath')};
					return this.findAll(function(e,i) {
										   if(e.options.selected) return e;
										   return false;
										}).collect(iterator);
				 },
				 
				 select: function() {
					this.each(function(e) {e.mark();});
                    this.options.selected = false;
					this.focus();
				 },

				 unselect: function() {
					this.each(function(e) {e.unmark();});
					this.focus();
				 },
				 
				 toString : function() {
					return '#[Tree' + this.children + ']'
				 },
				 
				 collapse : function() {
					this.each(function(e) {e.collapse();});
					this.focus();
				 },
				 
				 remove : function() {
					// remove selected
                    // here for selected we want the node, not the token
                    // so we pass the prototype ecKo 
					this.selected(Prototype.K).each(function(e) {e.remove()});
                    this.setCursor();
					
				 },
				 
				 expand : function() {
					this.each(function(e) {e.expand();});
					this.focus();
				 },

				 expanded : function(iterator) {
                    // find all the expanded nodes returning their token
                    // for possible use in a cookie
                    iterator = iterator || function(e) { return e.dispatch('getPath')};
					return this.findAll(function(e,i) {
										   if(e.options.expanded) return e;
										   return false;
										}).collect(iterator);

				 },
                 
                 //Cookie and path Management
                 walkPath : function (path) {
                    if (typeof path == "string") {
                       path = path.split('/');
                    }
                    var ob = this;
                    var p = 0; 
                    do {
                       ob = ob.walk(path[p]);
                       if (!ob) return null;
                       if (!ob.options.expanded) ob.expand();
                       p++;
                    } while (p < path.length);
                    return ob;
                 },
                 
                 loadCookie : function (name) {
                    var c = Cookie.get(name);
                    // expand each node and set the cursor 
                    this.collapse();
                    c.expanded.each(function(e){this.walkPath(e)}.bind(this));
                    // walking to the cursor implicitly expands it
                    // but I am ok with that.
                    this.setCursor(this.walkPath(c.cursor));
                 },
                 
                 storeCookie: function (name) {
                    // store a cookie under a given name 
                    // recording the current cursor and the 
                    // expansion paths of each node
                    var c = {};
                    c.expanded = this.expanded(Prototype.K).collect(function(e){return e.dispatch('getPath')});
                    c.cursor = this.cursor().dispatch('getPath');
                    Cookie.set(name, c);
                 },

                 // Enumeration support
				 _each : function(iterator) {
					if (this.children) {
					   this.children._each(function(e) {e._each(iterator)});
					}
					
				 },
                 
                 // Utils
                 generateId : function(prefix)  {
                    prefix = prefix || 'node';
                    return prefix + Math.round(1000000*Math.random());
                 },

                 clear : function() {
                    this.children = $A();
                    this.spawn.update('');
                 }

			  });


