import unittest
import sys, os
import difflib

# Conditional import of
base = ".."
try: base = os.path.abspath(os.path.join(os.path.dirname(__file__), base))
except: pass

sys.path.insert(0, base)
import pjs

    
class Test(unittest.TestCase):
    def test_element(self):
        assert isinstance(self.page['foo'], pjs.jselement)
        self.page['foo'].highlight()
        self.verify('$("foo").highlight();')

        el = self.page['element']
        el.show()
        self.verify('$("element").show();')

        self.page['element'].show("a", b="c")
        self.verify('$("element").show("a", {"b":"c"});')

        self.page['element'].visualEffect('highlight')
        self.verify('''$("element").visualEffect("highlight");''')

    def test_variable(self):
        page = self.page
        page.tree.addItem("foo")
        self.verify('var _v_1 = tree.addItem("foo");')

        # this test would try to json tree.selected(), a jsvariable
        # its tojs method would be called to return a reference
        page = self.page = pjs.JavascriptPage()
        tree = page.tree
        n = tree.walkPath(tree.selected())
        n.addItem("new")
        self.verify([
            'var _v_1 = tree.selected();',
            'var _v_2 = tree.selected(_v_1);',
            'var _v_3 = _v_2.addItem("new");',
            ])

##     def test_getattr(self):
##         page = self.page
##         tree = getattr(page, "tree")
##         x = tree.selected().first()
##         # here children isn't called, we want it to be an attribute of
##         # tree
##         # when an uncalled proxy is invoked we json the proxy name
##         tree.children().first().addItem('test')
##         assert "children" in str(page), """variable reference fail"""
        
    def test_jselement_arg(self):
        # here we pass a jselement as a argument
        # it creates a reference
        page = self.page
        folder = page.new('Tree')
        files = page.new('Tree')
        folder.render(page['folder'])
        files.render(page['file'])
        self.verify([
            'var tree1 = new Tree();',
            'var tree2 = new Tree();',
            '$("folder");',
            'var _v_1 = tree1.render($("folder"));',
            '$("file");',
            'var _v_2 = tree2.render($("file"));'
            ])

        
        
    def test_collection(self):
        s = self.page.select("p")
        # actually jscollectionelement
        assert isinstance(s, pjs.jscollection)
        s.each('highlight', delay=1.0)
        self.verify('$$("p").each(function(el) {el.highlight( {"delay":1.000000})};);')


    def test_page(self):
        page = self.page

        page.alert("testing")
        self.verify('alert("testing");')

        page.hide("foo", "bar")
        self.verify('Element.hide(["foo","bar"]);')

        page.show("foo", "bar")
        self.verify('Element.show(["foo","bar"]);')

        page.remove("error")
        self.verify('''["error"].each('Element.remove');''')

    def verify(self, expected, page=None):
        if not page: page = self.page
        
        try:
            if isinstance(expected, basestring):
                lines = expected.count("\n") or 1
            else:
                lines = len(expected)
            buf = page.lines[-lines:]
            if not isinstance(expected, list): expected = [expected]
            if len(buf) != lines: raise AssertionError
            if buf != expected: raise AssertionError
        except AssertionError:
            raise AssertionError("\n\t%s" % ('\n\t'.join(
                difflib.unified_diff(expected,
                                     buf,
                                     "expected",
                                     "generated",
                                     lineterm=""))
                                           ))
            
    def setUp(self):
        self.page = pjs.JavascriptPage()


def test_suite():
    from unittest import TestSuite, makeSuite
    import doctest
    suite = TestSuite()
    suite.addTest(makeSuite(Test))
    suite.addTest(doctest.DocFileSuite("../using_pjs.txt"))
    suite.addTest(doctest.DocTestSuite(pjs))
    return suite
    

if __name__ == "__main__":
    # in testing proxies its easy to get recursion problems
    # limit the stack depth to reduce output
    import sys
    sys.setrecursionlimit(25)
    
    runner = unittest.TextTestRunner(verbosity=1)
    runner.run(test_suite())
